/*
 * CompleteStrategy.java
 *
 * Created on April 17, 2004, 7:08 PM
 */

package simulations.core.strategy.completeStrategy;

import simulations.core.Player;
import simulations.core.history.Turn;
import simulations.core.exceptions.*;
import simulations.core.strategy.attackStrategy.AttackStrategy;
import simulations.core.strategy.askingSupportStrategy.AskingStrategy;
import simulations.core.strategy.decisionSupportStrategy.DecisionSupportStrategy;

import java.util.Vector;
import java.util.Iterator;
import java.util.ArrayList;

import java.io.Serializable;

/**
 * This abstract class describes haw to construct an attack strategy
 * @author  tom
 */
public abstract class CompleteStrategy extends simulations.core.strategy.Strategy implements Serializable, Cloneable {
    
    /** The attack strategies to use <b>in order</b> **/
    protected Vector attackStrategyVector;
    
    /** The asking strategies to use **/
    protected Vector askingStrategyVector;
    
    /** The decision strategies to use **/
    protected Vector decisionStrategyVector;
    
    /** Creates a new instance of CompleteStrategy */
    public CompleteStrategy() {
        super();
        attackStrategyVector = new Vector();
        askingStrategyVector = new Vector();
        decisionStrategyVector = new Vector();
    }
    
    /** Creates a new instance of CompleteStrategy
     * @param attackStrategyVector The attack strategies to use <b>in order</b>
     * @param askingStrategyVector The asking strategies to use
     * @param decisionStrategyVector The decision strategies to use
     **/
    public CompleteStrategy(Vector attackStrategyVector, Vector askingStrategyVector, Vector decisionStrategyVector ) {
        super();
        this.attackStrategyVector = attackStrategyVector;
        this.askingStrategyVector = askingStrategyVector;
        this.decisionStrategyVector = decisionStrategyVector;
    }
    
    /** return the player to attack
     * @param player The attacking player
     * @param playerList The list of <B>avalaible</B> player
     * @return The player to attack
     * @throws ManyPlayerToAttackException if this method would return more that one player to attack
     * @throws NoPlayerToAttackException if this method would return no player to attack
     */    
    public abstract Player getPlayerToAttack(Player player, ArrayList playerList) throws ManyPlayerToAttackException, NoPlayerToAttackException;
    
    /** Returns the list of players to ask support.
     * @return Returns the list of players to ask support.
     * @param attackedPlayer The attacked player
     * @param player The attacking player
     * @param playerList The list of <B>avalaible</B> player
     */    
    public abstract ArrayList getPlayersToAskSupport(Player player, Player attackedPlayer, ArrayList playerList);
    
    /** return true if the player will support the attack.
     * @return return true if the player will support the attack.
     * @param player The owner of the strategy
     * @param attackingPlayer The attacking player
     * @param attackedPlayer The attacked player
     * @param playerList The player list
     */    
    public abstract boolean supportAttack(Player player, Player attackingPlayer, Player attackedPlayer, ArrayList playerList);
    
    /**
     * This method is called by the judge after the decision of players to attack and to ask supports
     * to inform the strategy
     * @param player The owner of the strategy
     * @param attacksList The list of attacks
     */
    public abstract void updateAttacksInfos(Player player, ArrayList attacksList);
    
    /** This method is called by the judge to inform players of the last turn
     * @param turn The turn
     * @param player The owner of the strategy
     * @param remainingPlayers The list of the remaining players
     */    
    public abstract void updateLastTurn(Turn turn, Player player, ArrayList remainingPlayers);
    
    /** return the description of this strategy
     * @return return the description of this strategy
     **/
    public abstract String getDescription();
    
    /** Getter for property askingStrategyVector.
     * @return Value of property askingStrategyVector.
     *
     */
    public java.util.Vector getAskingStrategyVector() {
        return askingStrategyVector;
    }
    
    /** Setter for property askingStrategyVector.
     * @param askingStrategyVector New value of property askingStrategyVector.
     *
     */
    public void setAskingStrategyVector(java.util.Vector askingStrategyVector) {
        this.askingStrategyVector = askingStrategyVector;
    }
    
    /** Getter for property attackStrategyVector.
     * @return Value of property attackStrategyVector.
     *
     */
    public java.util.Vector getAttackStrategyVector() {
        return attackStrategyVector;
    }
    
    /** Setter for property attackStrategyVector.
     * @param attackStrategyVector New value of property attackStrategyVector.
     *
     */
    public void setAttackStrategyVector(java.util.Vector attackStrategyVector) {
        this.attackStrategyVector = attackStrategyVector;
    }
    
    /** Getter for property decisionStrategyVector.
     * @return Value of property decisionStrategyVector.
     *
     */
    public java.util.Vector getDecisionStrategyVector() {
        return decisionStrategyVector;
    }
    
    /** Setter for property decisionStrategyVector.
     * @param decisionStrategyVector New value of property decisionStrategyVector.
     *
     */
    public void setDecisionStrategyVector(java.util.Vector decisionStrategyVector) {
        this.decisionStrategyVector = decisionStrategyVector;
    }
    
    /** overload of equals method
     * @param o The object to compare
     * @return Returns true if this and o are equals
     **/
    public boolean equals(Object o) {
        if (o instanceof CompleteStrategy) {
            if (name.equals(((CompleteStrategy)o).getName())) {
                return true;
            }
        }
        return false;
    }
    
    /** Return the composition of the attack list 
     * @return the composition of the attack list 
     **/
    public String getAttackComposition() {
        String str = "";
        Iterator it = attackStrategyVector.iterator();
        int cpt=1;
        while (it.hasNext()) {
            AttackStrategy strat = (AttackStrategy)it.next();
            str=str+"#"+cpt+" : "+strat.getName()+" "+
            negativeToString(strat.isNegative())+" memory = "+strat.getMemory()+"\n";
            cpt++;
        }
        return str;
    }
    
    /** Returns a string representation of negative
     * @return  a string representation of negative
     **/
    private String negativeToString(boolean b) {
        if (b) {
            return "NEGATIVE";
        } else {
            return "NON-NEGATIVE";
        }
    }
    
    /** Return the composition of the asking list 
     * @return the composition of the asking list 
     **/
    public String getAskingComposition() {
        String str = "";
        Iterator it = askingStrategyVector.iterator();
        int cpt=1;
        while (it.hasNext()) {
            AskingStrategy strat = (AskingStrategy)it.next();
            str=str+"#"+cpt+" : "+strat.getName()+" "+
            negativeToString(strat.isNegative())+" memory = "+strat.getMemory()+"\n";
            cpt++;
        }
        return str;
    }
    
    /** Return the composition of the decision list 
     * @return the composition of the decision list 
     **/
    public String getDecisionComposition() {
        String str = "";
        Iterator it = decisionStrategyVector.iterator();
        int cpt=1;
        while (it.hasNext()) {
            DecisionSupportStrategy strat = (DecisionSupportStrategy)it.next();
            str=str+"#"+cpt+" : "+strat.getName()+" "+
            negativeToString(strat.isNegative())+" memory = "+strat.getMemory()+"\n";
            cpt++;
        }
        return str;
    }
    
}
