/*
 * coalitionCanvas.java
 *
 * Created on April 12, 2004, 1:37 AM
 */

package simulations.gui.coalitions;

import java.awt.Color;
import java.awt.Graphics;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.Vector;
import java.util.Random;
import java.util.GregorianCalendar;
import java.util.Calendar;

import simulations.core.Player;
import simulations.core.strategy.Strategy;
import simulations.gui.curves.ColorElement;

/** This class describes the canvas containing the coalitions graph
 * @author tom
 */
public class CoalitionCanvas extends java.awt.Canvas {
    
    /** This vector represents the grid : each cell is a player **/
    private Color[][] table;
    
    /** The coalition list **/
    private ArrayList coalitionList;
    
    /** The size **/
    private int dim;
    
    /** The colors player vector **/
    private Vector colorsPlayerVector;
    
    /** The colors strategy vector **/
    private Vector colorsStrategyVector;
    
    /** The strategy list **/
    private ArrayList strategyList;
    
    /** Creates a new instance of coalitionCanvas
     * @param playerList The players list
     * @param size The number of players
     */
    public CoalitionCanvas(ArrayList playerList, int size) {
        table=new Color[size][size];
        coalitionList = new ArrayList();
        colorsPlayerVector = new Vector();
        colorsStrategyVector = new Vector();
        colorDefinition(playerList);
        this.dim=size;
        this.setBackground(new java.awt.Color(255, 255, 255));
        this.setSize(4*dim,4*dim);
    }
    
    /** returns the color associate to a player
     * @param p The player
     * @return returns the color associate to player p
     **/
    private Color getColor(Player p) {
        Iterator it = colorsPlayerVector.iterator();
        while (it.hasNext()) {
            ColorElement elt = (ColorElement)it.next();
            if (((Player)elt.getObject()).equals(p)) {
                return elt.getColor();
            }
        }
        return null;
    }
    
    /** returns the color associate to a strategy
     * @param s The strategy
     * @return returns the color associate to strategy s
     **/
    private Color getColor(Strategy s) {
        Iterator it = colorsStrategyVector.iterator();
        while (it.hasNext()) {
            ColorElement elt = (ColorElement)it.next();
            if (((Strategy)elt.getObject()).equals(s)) {
                return elt.getColor();
            }
        }
        return null;
    }
    
    /** Def a color by strategy and then a color by player (with same strategy)
     * @param playerList The player list
     */
    public void colorDefinition(ArrayList playerList) {
        // CAUTION : Composed strategy defines more strategy!!!
        strategyList = new ArrayList();
        colorsStrategyVector.removeAllElements();
        Iterator it = playerList.iterator();
        while (it.hasNext()) {
            Player p = (Player)it.next();
            Strategy strat = (Strategy)p.getStrategy();
            if (!(strategyList.contains(strat))) {
                strategyList.add(strat);
            }
        }
        if (simulations.SimulationParameters.debug) {
            System.out.println("StrategyList = ");
            System.out.println(strategyList.toString());
        }
        it = strategyList.iterator();
        Random random = new Random(new GregorianCalendar().get(Calendar.MILLISECOND));
        int index=0;
        while (it.hasNext()) {
            Strategy strat = (Strategy)it.next();
            switch (index) {
                case 0 :
                    colorsStrategyVector.add(new ColorElement(strat, Color.BLACK));
                    break;
                case 1 : 
                    colorsStrategyVector.add(new ColorElement(strat, Color.BLUE));
                    break;
                case 2 : 
                    colorsStrategyVector.add(new ColorElement(strat, Color.GRAY));
                    break;
                case 3 : 
                    colorsStrategyVector.add(new ColorElement(strat, Color.GREEN));
                    break;
                case 4 : 
                    colorsStrategyVector.add(new ColorElement(strat, Color.MAGENTA));
                    break;
                case 5 : 
                    colorsStrategyVector.add(new ColorElement(strat, Color.ORANGE));
                    break;
                case 6 : 
                    colorsStrategyVector.add(new ColorElement(strat, Color.PINK));
                    break;
                case 7 : 
                    colorsStrategyVector.add(new ColorElement(strat, Color.YELLOW));
                    break;
                default :
                    colorsStrategyVector.add(new ColorElement(strat, new Color(random.nextInt(256),random.nextInt(256),random.nextInt(256))));
                    break;
            }
            index++;
        }
        it = playerList.iterator();
        while (it.hasNext()) {
            Player p = (Player)it.next();
            colorsPlayerVector.add(new ColorElement(p, getColor(p.getStrategy())));
        }
    }
    
    /** Draw the grid
     * @param g The graphics
     */
    public void drawGrid(Graphics g) {
        int size = 4*dim;
        for(int i=0;i<dim;i++) {
            g.drawLine(0, 4*i, size, 4*i);
            g.drawLine(4*i, 0, 4*i,size);
        }
        g.drawLine(0, size, size, size);
        g.drawLine(size, 0, size, size);
    }
    
    /** This method draw a filled cube at point (x,y) with a color
     * @param g The graphics
     * @param x The x point to begin
     * @param y The y point to begin
     * @param color The color to use to draw
     */    
    public void drawCube(Graphics g, int x, int y, Color color) {
        g.setColor(color);
        g.drawLine(x, y, x+2, y);
        g.drawLine(x,y+1, x+2, y+1);
        g.drawLine(x,y+2, x+2, y+2);
    }
    
    /** Overload of paint method
     * @param g The graphics
     */
    public void paint(Graphics g) {
        drawGrid(g);
        // each line is a coalition
        Iterator it = coalitionList.iterator();
        int y=1;
        while (it.hasNext()) {
            // We have the first list
            Iterator it2 = ((ArrayList)it.next()).iterator();
            int x=1;
            while (it2.hasNext()) {
                // draw each player
                drawCube(g, x, y, getColor((Player)it2.next()));
                x+=4;
            }
            y+=4;
        }
    }
    
    /** Getter for property coalitionList.
     * @return Value of property coalitionList.
     *
     */
    public java.util.ArrayList getCoalitionList() {
        return coalitionList;
    }
    
    /** Setter for property coalitionList.
     * @param coalitionList New value of property coalitionList.
     *
     */
    public void setCoalitionList(java.util.ArrayList coalitionList) {
        this.coalitionList = coalitionList;
        //System.out.println("Coalition list = ");
        //System.out.println(coalitionList);
    }
    
    /** Getter for property colorsPlayerVector.
     * @return Value of property colorsPlayerVector.
     *
     */
    public java.util.Vector getColorsPlayerVector() {
        return colorsPlayerVector;
    }
    
    /** Setter for property colorsPlayerVector.
     * @param colorsPlayerVector New value of property colorsPlayerVector.
     *
     */
    public void setColorsPlayerVector(java.util.Vector colorsPlayerVector) {
        this.colorsPlayerVector = colorsPlayerVector;
    }
    
    /** Getter for property colorsStrategyVector.
     * @return Value of property colorsStrategyVector.
     *
     */
    public java.util.Vector getColorsStrategyVector() {
        return colorsStrategyVector;
    }
    
    /** Setter for property colorsStrategyVector.
     * @param colorsStrategyVector New value of property colorsStrategyVector.
     *
     */
    public void setColorsStrategyVector(java.util.Vector colorsStrategyVector) {
        this.colorsStrategyVector = colorsStrategyVector;
    }
    
}
